package week03

import org.scalatest.concurrent.{Signaler, TimeLimitedTests}
import org.scalatest.flatspec.AnyFlatSpec
import org.scalatest.time.{Millis, Span}

import scala.util.Random

class PracticeTest extends AnyFlatSpec with TimeLimitedTests {
  val timeLimit: Span = Span(1000, Millis)
  override val defaultTestSignaler: Signaler = new Signaler {
    override def apply(testThread: Thread): Unit = {
      println("Ez a teszt túl sokáig fut.")
      testThread.stop()
    } //unsafe, never használd.
  }

  def generateIntList(length: Int, range: Int, positives: Boolean):List[Int] = Random.nextInt(length) match {
    case 0 => Nil
    case _ => {
      val head = Random.nextInt(range);
      (if(positives) head else (head - range/2))::generateIntList(length, range, positives)
    }
  }

  def find(list: List[Int], value: Int): Int = {
    def find(list: List[Int], value: Int, base: Int): Int = list match {
      case Nil => -1
      case head :: _ if (head == value ) => base
      case _ :: tail => find(tail, value, base+1)
    }
    find(list,value,0)
  }

  "find" should "return the index of the element found" in {
    for (i<-1 to 1000) {
      val input = generateIntList(20,10,true)
      val value = Random.nextInt(10)
      assert(Practice.find(input,value) == find(input,value), "testcase: list = " + input + ", value = " + value)
    }
  }

  "contains" should "return whether the element is found in the list" in {
    for(i<-1 to 1000) {
      val input = generateIntList(20,10,true)
      val value = Random.nextInt(10)
      assert(Practice.containsValue(input,value) == input.contains(value), "testcase: list = " + input + ", value = " + value)
    }
  }

  def removeDuplicates(list: List[Int]): List[Int] = {
    def collectFirst(list: List[Int], acc: List[Int]): List[Int] = list match {
      case Nil => acc
      case head :: tail => if(acc.contains(head)) collectFirst(tail,acc) else collectFirst(tail,head::acc)
    }
    collectFirst(list,Nil).reverse
  }

  "removeDuplicates" should "remove the duplicates" in {
    for(i<-1 to 1000) {
      val list = generateIntList(20,10,true)
      assert(Practice.removeDuplicates(list) == removeDuplicates(list), "testcase: list = " + list)
    }
  }

  "sumSquares" should "sum the squares" in {
    for(i<-1 to 1000) {
      assert(Practice.sumSquares(i) == (i*(i+1)*(2*i+1)/6), "testcase: i = " + i)
    }
  }

  "squareOdds" should "square the odd numbers" in {
    for(i<-1 to 1000) {
      val list = generateIntList(20,10,true)
      assert(Practice.squareOdds(list) == (list map { x => if (x%2==0) x else x*x }), "testcase: list = " + list)
    }
  }

  def isPrime(n: Int): Boolean = {
    def isPrime(m: Int): Boolean = {
      if(m*m > n) true else if( n % m == 0 ) false else isPrime(m+1)
    }
    if( n <= 1 ) false else isPrime(2)
  }
  "isPrime" should "test for primes" in {
    for(i<-1 to 1000) {
      assert(Practice.isPrime(i) == isPrime(i), "testcase: i = " + i)
    }
  }

  "splitPrimes" should "split the input list in two based on primality" in {
    for(i<-1 to 1000) {
      val list = generateIntList(20,10,false)
      assert(Practice.splitPrimes(list) == (list.filter{ _>1 } partition { isPrime _ }), "testcase: list = " + list)
    }
  }

}

